part of 'dip_nav_router.dart';

/// A delegate that is used by the [Router] widget to build and configure a
/// navigating widget.
class _DipNavDelegate extends RouterDelegate<DipNavRoute>
    with ChangeNotifier, PopNavigatorRouterDelegateMixin<DipNavRoute> {
  /// The list of [ResultablePage]
  final List<ResultablePage<DipNavRoute>> _pages = [];

  /// The list of [DipNavRoute]
  final List<DipNavRoute> routes = [];

  /// The [DipNavRoute]
  final DipNavRoute initRoute;

  /// The [DipNavRoute]
  final DipNavRoute? notFoundRoute;

  _DipNavDelegate({required this.initRoute, this.notFoundRoute});

  @override
  Widget build(BuildContext context) {
    if (_pages.isEmpty) {
      _addPageData(initRoute);
    }
    return Navigator(
      key: navigatorKey,
      onPopPage: _onPopPage,
      pages: List.of(_pages),
    );
  }

  @override
  GlobalKey<NavigatorState> get navigatorKey => GlobalKey<NavigatorState>();

  @override
  Future<void> setNewRoutePath(DipNavRoute configuration) {
    _pages.clear();
    return pushRoute(configuration);
  }

  @override
  DipNavRoute get currentConfiguration {
    final route = _pages.last.arguments;
    return (route != null && route is DipNavRoute) ? route : initRoute;
  }

  Future<T?> pushRoute<T>(DipNavRoute route,
      {bool removeUntil = false, bool replaceCurrent = false}) async {
    assert(!(removeUntil && replaceCurrent),
        'Only removeUntil or replaceCurrent should by true!');
    if (removeUntil) {
      _pages.clear();
    } else if (replaceCurrent && _pages.isNotEmpty) {
      _pages.removeLast();
    }
    _addPageData(route);
    notifyListeners();
  }

  @override
  Future<bool> popRoute() {
    if (_pages.length > 1) {
      _removePage(_pages.last);
      return Future.value(true);
    }
    return Future.value(false);
  }

  Future<DipNavRoute?> _findRoutByPath(String routePath) async {
    DipNavRoute? foundRoute;
    for (var route in routes) {
      if (route.path == routePath) {
        foundRoute = route;
        break;
      }
    }
    return Future.value(foundRoute);
  }

  void setPath(List<ResultablePage<DipNavRoute>> path) {
    _pages.clear();
    _pages.addAll(path);
    notifyListeners();
  }

  bool _onPopPage(Route<dynamic> route, result) {
    final didPop = route.didPop(result);
    if (!didPop) {
      return false;
    }
    _pages.remove(route.settings);
    notifyListeners();
    return true;
  }

  void _removePage(ResultablePage<DipNavRoute> page) {
    _pages.remove(page);
    notifyListeners();
  }

  ResultablePage<DipNavRoute> _createPage(DipNavRoute pageConfig) {
    return ResultablePage<DipNavRoute>(
        child: pageConfig.child,
        key: ValueKey<String>(pageConfig.key ?? pageConfig.path),
        path: pageConfig.path,
        name: pageConfig.path,
        arguments: pageConfig);
  }

  void _addPageData(DipNavRoute pageConfig) {
    _pages.add(
      _createPage(pageConfig),
    );
  }
}

/// A class of configuration [PageRoute]
class DipNavRoute {
  final String? key;
  final String path;
  final Widget child;
  final String? name;
  final Object? arguments;

  const DipNavRoute({
    this.key,
    required this.path,
    required this.child,
    this.name,
    this.arguments,
  });
}

/// A page that creates a material style [PageRoute].
///
/// {@macro flutter.material.materialRouteTransitionMixin}
class ResultablePage<T> extends MaterialPage<T> {
  ResultablePage({
    required Widget child,
    required String path,
    bool maintainState = true,
    bool fullscreenDialog = false,
    LocalKey? key,
    String? name,
    Object? arguments,
  }) : super(
            child: child,
            maintainState: maintainState,
            fullscreenDialog: fullscreenDialog,
            key: key,
            name: path,
            arguments: arguments);
}
